/* -------------------------------------------------------
                         onoff.c

     Demo fuer ein/ausschalten der  Betriebsspannung
     ueber einen einzelnen Taster

     MCU      :  STM8S103F3
     IDE      :  keine (Makefileprojekt)
     Compiler : SDCC 3.5
     Takt     :  interner Takt 16 MHz

     14.12.2016  R. Seelig
   ------------------------------------------------------ */

/*
##################################################################
                      STM8S103F3P6 Pinout
##################################################################


                            ------------
UART1_CK / TIM2_CH1 / PD4  |  1     20  |  PD3 / AIN4 / TIM2_CH2 / ADC_ETR
    UART1_TX / AIN5 / PD5  |  2     19  |  PD2 / AIN3
    UART1_RX / AIN6 / PD6  |  3     18  |  PD1 / SWIM
                     NRST  |  4     17  |  PC7 / SPI_MISO
              OSCIN / PA1  |  5     16  |  PC6 / SPI_MOSI
             OSCOUT / PA2  |  6     15  |  PC5 / SPI_CLK
                Vss (GND)  |  7     14  |  PC4 / TIM1_CH4 / CLK_CCO / AIN2
                VCAP (*1)  |  8     13  |  PC3 / TIM1_CH3 /
                Vdd (+Ub)  |  9     12  |  PB4 / I2C_SCL
           TIM2_CH3 / PA3  | 10     11  |  PB5 / I2C_SDA
                            -----------

*1 :  Ist mit min. 1uF gegen GND zu verschalten

*/

/*
##################################################################
                            Schaltung
##################################################################

                                      + ---------+
                 BC 559               | AMS 1117 |
 +5V o---+--------    ---------+------|   3.3    |------+------------+----------------+---o 3.3V out
         |        >  /         |      +----------+      |            |                |
         |        ---         ---          |           ---           |                |
         |         |          ---          |           ---           |                |
       \          +-+ 4.7k     | 100n      |            | 100n       |                |
        \         | |         ---         ---          ---           |                |
  PWR    |        +-+                                                |                |
 Button  |         |     1N4148                                      +----+           |
         +---------------- >|----+                                   |    |           |
         |         |             |                                   |   ---         +-+
         V  1N     |             |                                   |   ---         | |
         -  4148   |             |                                   |    | 100n     +-+
         |         |             |                                   |   ---          | 100k
        +-+ 18k    |             |               STM8S103          9 |                |
        | |        |             |           +---------------------------------+      |
        +-+        |             |           |                      Vdd        |      |
         |         |             |           |                                 | 4    |
         +-----------------------------------|  PD4 (pwbutton)             res |------+
         |         |             |        20 |                                 |
        +-+ 33k     \|    ___    |           |                                 |
        | |          |---|___|---+-----------|  PD3 (hold)                     |
        +-+     BC  <|   4.7k              1 |                                 |
         |     549 |                         |                                 |
         |         |                         +---------------------------------+
         |         |                                       7 |     8 |
         |         |                                         |       | +
         |         |                                         |      ---
         |         |                                         |      --- 1u
         |         |                                         |       |
        ---       ---                                       ---     ---
*/


#include "stm8s.h"
#include "stm8_init.h"
#include "stm8_gpio.h"

#define blinkspeed        100

#define led1_init()       PB5_output_init()
#define led1_set()        PB5_clr()
#define led1_clr()        PB5_set()

#define hold_init()       PD3_output_init()
#define hold_set()        PD3_set()
#define hold_clr()        PD3_clr()

#define pwbutton_init()   PD4_input_init()      // Powerbutton
#define is_pwbutton()     is_PD4()


uint16_t ledtim = 0;

/* --------------------------------------------------
               Timer1 overflow - interrupt

    wird jede Millisekunde aufgerufen (Initialisierung
    in tim1_init).
    -------------------------------------------------- */
void tim1_ovf(void) __interrupt(11)
// 11 ist der Interruptvektor fuer Timer1 overflow
{
  // Benutzercode hier
  ledtim++;


  // Abschalten?
  if (is_pwbutton())
  {
    int_disable();                  // Interrupt aus fuer "runterfahren"
    delay_ms(50);                   // normalerweise in ISR verboten, aber
                                    // ab hier wird die MCU sowieso ausgeschaltet
    while(is_pwbutton());
    delay_ms(100);
    hold_clr();                     // Selbsthaltung und somit Betriebsspg. aus
                                    // hierher kommt das Programm nie
  }

  TIM1_SR1 &= ~TIM1_SR1_UIF;        // Interrupt quittieren
}


/* --------------------------------------------------
                       tim1_init
      intialisiert Timer mit Taktteiler / 16000
      (Prescaler somit = 15999) und startet diesen im
      Autoreloadmodus mit Autoreloadwert 999 (jeder
      1000ste Impuls erzeugt Interrupt)

      Bei F_CPU = 16 MHz gilt:

      16 MHz / 16 = 1 MHz => 1 us
      Autoreloadwert 999 => Wert / 1000

      => 1 us / 1000 = 1 ms !!!!!
   -------------------------------------------------- */
void tim1_init(void)
{

  // Prescaler-Register = 15 entspricht Teilung / 16 .. ( 0..15 = 16 Schritte)
  // Timer wird somit bei F_CPU= 16 MHz mit 1uS getaktet
  #define clockdivisor     ((uint16_t) 15)

  TIM1_PSCRH= (uint8_t) (clockdivisor >> 8);
  TIM1_PSCRL= (uint8_t) (clockdivisor & 0x0000ff);

  // Enable overflow
  TIM1_IER |= TIM1_IER_UIE;

  // Timer1 starten
  TIM1_CR1 |= TIM1_CR1_CEN;     // Timer1 enable ( CEN = ClockENable )
  TIM1_CR1 |= TIM1_CR1_ARPE;    // Timer1 autoreload ( mit Werten in TIM1_ARR )

  // Autoreloadwert = 999 (jeder 1000ste Impuls ein Interrupt)
  TIM1_ARRH= (uint8_t) (999 >> 8);
  TIM1_ARRL= (uint8_t) (999 & 0x00ff);

  int_enable();
}


int main(void)
{
  sysclock_init(0);                     // System mit 16 MHz interner Quarz

  hold_init();
  hold_set();                           // Controller geht in  "Selbsthaltung"

  led1_init();
  pwbutton_init();

  delay_ms(50);
  led1_clr();
  while(is_pwbutton());
  delay_ms(50);
  tim1_init();

  while(1)
  {
    if (ledtim> blinkspeed) led1_set();
    if (ledtim> (blinkspeed*2)) { led1_clr(); ledtim= 0; }
  }
}

